#!/usr/bin/env python2
#
# larchin.py - The larchin live Arch Linux installer
#
# (c) Copyright 2010 Michael Towers (larch42 at googlemail dot com)
#
# This file is part of the larch project.
#
#    larch is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    larch is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with larch; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#----------------------------------------------------------------------------
#2010.08.15

import sys, os
dirpath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(dirpath + '/front')
base_dir = os.path.dirname(dirpath)
sys.path.append(base_dir + '/cli')
sys.path.append(os.path.dirname(base_dir))

from controller import exports, Fs
from mainwindow import Ui


def backend(func, *args):
    """Interface between GUI and file-system accessing functions (normally
    the main, functional part of the application). The first argument is the
    function name. There may be any number of arguments, including none.
    The simplest functions, those completing quickly and without interaction,
    return a (<bool> ok, result) pair - the type of the result is open.
    Longer running functions, or those that require some interaction return
    <None>, and then communicate subsequently using 'signals'.
    """
    return exports[func](*args)


if __name__ == "__main__":
    ## Note that the gui module must have a reference point for relative
    ## paths, so the current directory must be set to the larch base directory
    ## before starting the gui:
    os.chdir(base_dir)

    # Initialize the backend
    app = Fs()
    add_exports(app)
    # Initialize the GUI
    ui = Ui(backend)
    # Tell the backend how to send signals to the GUI
    app.setuisig(ui.sigin)
    # Start the application, enter the main event loop
    ui.init()

